/**
 * 
 */
package ch.psi.daq.queryrest.controller;

import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.Set;

import javax.annotation.Resource;

import org.springframework.validation.Errors;
import org.springframework.validation.Validator;

import ch.psi.daq.query.model.AbstractQuery;
import ch.psi.daq.query.model.AbstractTimeRangeQuery;
import ch.psi.daq.query.model.Aggregation;
import ch.psi.daq.query.model.DBMode;
import ch.psi.daq.query.model.QueryField;
import ch.psi.daq.query.range.QueryRange;
import ch.psi.daq.queryrest.config.QueryRestConfig;

/**
 * @author zellweger_c
 *
 */
public class QueryValidator implements Validator {

   @Resource(name = QueryRestConfig.BEAN_NAME_DEFAULT_RESPONSE_FIELDS)
   private Set<QueryField> defaultResponseFields;

   @Resource(name = QueryRestConfig.BEAN_NAME_DEFAULT_RESPONSE_AGGREGATIONS)
   private Set<Aggregation> defaultResponseAggregations;

   /**
    * {@inheritDoc}
    */
   @Override
   public boolean supports(Class<?> clazz) {
      return AbstractQuery.class.isAssignableFrom(clazz);
   }

   /**
    * {@inheritDoc}
    */
   @Override
   public void validate(Object target, Errors errors) {

      AbstractQuery query = (AbstractQuery) target;

      QueryRange queryRange = query.getQueryRange();

      if (!queryRange.isPulseIdRangeDefined() && !queryRange.isTimeRangeDefined()) {
         errors.reject("", "Either time or pulseId range must be defined by the query!");
      }

      if (queryRange.isPulseIdRangeDefined()) {
         if (queryRange.getStartPulseId() > queryRange.getEndPulseId()) {
            errors.rejectValue("pulseId", "pulseId", "Start pulse-id must be smaller or equals end pulse-id!");
         }
      } else if (queryRange.isTimeRangeDefined()) {
         if (queryRange.getStartMillis() > queryRange.getEndMillis()) {
            errors.rejectValue("millis", "millis", "Start millis must be smaller or equals to end millis!");
         } else if (queryRange.getStartMillis() == queryRange.getEndMillis()
               && queryRange.getStartNanos() > queryRange.getEndNanos()) {
            errors.rejectValue("nanos", "nanos", "Start nanos must be smaller or equals to end nanos!");
         }
      }

      if (DBMode.archiverappliance.equals(query.getDBMode())) {
         if (!(query instanceof AbstractTimeRangeQuery)) {
            errors.reject("", "ArchiverAppliance supports time range queries only!");
         }
      }

      if (query.getFields() == null || query.getFields().isEmpty()) {
         query.setFields(new LinkedHashSet<>(defaultResponseFields));
      }

      if (query.getAggregations() == null || query.getAggregations().isEmpty()) {
         query.setAggregations(new LinkedList<>(defaultResponseAggregations));
      }
   }

}
